@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
+# Copyright 2016-2018 Julien Danjou
 # Copyright 2017 Elisey Zanko
 # Copyright 2016 Étienne Bersac
-# Copyright 2016 Julien Danjou
 # Copyright 2016 Joshua Harlow
 # Copyright 2013-2014 Ray Holder
 #
@@ -78,6 +78,10 @@ from .before import before_nothing  # noqa
 from .after import after_log  # noqa
 from .after import after_nothing  # noqa
 
+# Import all built-in after strategies for easier usage.
+from .before_sleep import before_sleep_log  # noqa
+from .before_sleep import before_sleep_nothing  # noqa
+
 
 def retry(*dargs, **dkw):
     """Wrap a function with a new `Retrying` object.
@@ -144,14 +148,17 @@ class BaseRetrying(object):
                  retry=retry_if_exception_type(),
                  before=before_nothing,
                  after=after_nothing,
+                 before_sleep=before_sleep_nothing,
                  reraise=False,
-                 retry_error_cls=RetryError):
+                 retry_error_cls=RetryError,
+                 retry_error_callback=None):
         self.sleep = sleep
         self.stop = stop
         self.wait = wait
         self.retry = retry
         self.before = before
         self.after = after
+        self.before_sleep = before_sleep
         self.reraise = reraise
         self._local = threading.local()
         # This will allow for passing in the result and handling
@@ -159,10 +166,14 @@ class BaseRetrying(object):
         # the prior result.
         self._wait_takes_result = self._waiter_takes_last_result(wait)
         self.retry_error_cls = retry_error_cls
+        self.retry_error_callback = retry_error_callback
 
     def copy(self, sleep=_unset, stop=_unset, wait=_unset,
-             retry=_unset, before=_unset, after=_unset, reraise=_unset):
+             retry=_unset, before=_unset, after=_unset, before_sleep=_unset,
+             reraise=_unset):
         """Copy this object with some parameters changed if needed."""
+        if before_sleep is _unset:
+            before_sleep = self.before_sleep
         return self.__class__(
             sleep=self.sleep if sleep is _unset else sleep,
             stop=self.stop if stop is _unset else stop,
@@ -170,6 +181,7 @@ class BaseRetrying(object):
             retry=self.retry if retry is _unset else retry,
             before=self.before if before is _unset else before,
             after=self.after if after is _unset else after,
+            before_sleep=before_sleep,
             reraise=self.reraise if after is _unset else reraise,
         )
 
@@ -278,6 +290,8 @@ class BaseRetrying(object):
             delay_since_first_attempt
         if self.stop(self.statistics['attempt_number'],
                      delay_since_first_attempt):
+            if self.retry_error_callback:
+                return self.retry_error_callback(fut)
             retry_exc = self.retry_error_cls(fut)
             if self.reraise:
                 raise retry_exc.reraise()
@@ -295,6 +309,9 @@ class BaseRetrying(object):
         self.statistics['idle_for'] += sleep
         self.statistics['attempt_number'] += 1
 
+        if self.before_sleep is not None:
+            self.before_sleep(self, sleep=sleep, last_result=fut)
+
         return DoSleep(sleep)
 
 
@@ -352,7 +369,7 @@ class Future(futures.Future):
 
 
 if asyncio:
-    from tenacity.async import AsyncRetrying
+    from tenacity._asyncio import AsyncRetrying
 
 if tornado:
     from tenacity.tornadoweb import TornadoRetrying
